@@ -1,32 +1,32 @@
 /*----------------------------------------------------------------------------
-    ChucK Concurrent, On-the-fly Audio Programming Language
-      Compiler and Virtual Machine
+  ChucK Concurrent, On-the-fly Audio Programming Language
+    Compiler and Virtual Machine
 
-    Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
-      http://chuck.cs.princeton.edu/
-      http://soundlab.cs.princeton.edu/
+  Copyright (c) 2004 Ge Wang and Perry R. Cook.  All rights reserved.
+    http://chuck.stanford.edu/
+    http://chuck.cs.princeton.edu/
 
-    This program is free software; you can redistribute it and/or modify
-    it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
-    (at your option) any later version.
+  This program is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
 
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
-    You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-    U.S.A.
+  You should have received a copy of the GNU General Public License
+  along with this program; if not, write to the Free Software
+  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+  U.S.A.
 -----------------------------------------------------------------------------*/
 
 //-----------------------------------------------------------------------------
 // file: uana_xform.cpp
-// desc: ...
+// desc: unit analyzer transforms
 //
-// author: Ge Wang (gewang@cs.princeton.edu)
+// author: Ge Wang (ge@ccrma.stanford.edu | gewang@cs.princeton.edu)
 //         Rebecca Fiebrink (fiebrink@cs.princeton.edu)
 // date: Spring 2007
 //-----------------------------------------------------------------------------
@@ -936,7 +936,7 @@ CK_DLL_CTRL( FFT_ctrl_size )
 invalid_size:
     // we have a problem
     fprintf( stderr, 
-        "[chuck](IFFT): InvalidTransformSizeException (%d)\n", size );
+        "[chuck](IFFT): InvalidTransformSizeException (%ld)\n", size );
     goto done;
 
 done:
@@ -1469,7 +1469,7 @@ CK_DLL_CTRL( IFFT_ctrl_size )
 invalid_size:
     // we have a problem
     fprintf( stderr, 
-        "[chuck](IFFT): InvalidTransformSizeException (%d)\n", size );
+        "[chuck](IFFT): InvalidTransformSizeException (%ld)\n", size );
     goto done;
 
 done:
@@ -1556,7 +1556,7 @@ static t_CKBOOL prepare_window( void * ARGS, Chuck_VM_Shred * SHRED, t_CKINT & s
 out_of_memory:
     // we have a problem
     fprintf( stderr, 
-        "[chuck](Windowing): OutOfMemoryException (allocating FLOAT[%d])\n",
+        "[chuck](Windowing): OutOfMemoryException (allocating FLOAT[%ld])\n",
         float_array_size );
     goto done;
 
@@ -2111,7 +2111,7 @@ CK_DLL_CTRL( Flip_ctrl_size )
 invalid_size:
     // we have a problem
     fprintf( stderr, 
-        "[chuck](IFFT): InvalidTransformSizeException (%d)\n", size );
+        "[chuck](IFFT): InvalidTransformSizeException (%ld)\n", size );
     goto done;
 
 done:
@@ -2620,7 +2620,7 @@ CK_DLL_CTRL( UnFlip_ctrl_size )
 invalid_size:
     // we have a problem
     fprintf( stderr, 
-        "[chuck](UnFlip): InvalidTransformSizeException (%d)\n", size );
+        "[chuck](UnFlip): InvalidTransformSizeException (%ld)\n", size );
     goto done;
 
 done:
@@ -3033,9 +3033,9 @@ CK_DLL_TOCK( DCT_tock )
 CK_DLL_MFUN( DCT_transform )
 {
     // get object
-    DCT_object * dct = (DCT_object *)OBJ_MEMBER_UINT(SELF, DCT_offset_data);
+//    DCT_object * dct = (DCT_object *)OBJ_MEMBER_UINT(SELF, DCT_offset_data);
     // get array
-    Chuck_Array8 * arr = (Chuck_Array8 *)GET_NEXT_OBJECT(ARGS);
+//    Chuck_Array8 * arr = (Chuck_Array8 *)GET_NEXT_OBJECT(ARGS);
 }
 
 
@@ -3098,7 +3098,7 @@ CK_DLL_CTRL( DCT_ctrl_size )
 invalid_size:
     // we have a problem
     fprintf( stderr, 
-        "[chuck](IDCT): InvalidTransformSizeException (%d)\n", size );
+        "[chuck](IDCT): InvalidTransformSizeException (%ld)\n", size );
     goto done;
 
 done:
@@ -3627,7 +3627,7 @@ CK_DLL_CTRL( IDCT_ctrl_size )
 invalid_size:
     // we have a problem
     fprintf( stderr, 
-        "[chuck](IDCT): InvalidTransformSizeException (%d)\n", size );
+        "[chuck](IDCT): InvalidTransformSizeException (%ld)\n", size );
     goto done;
 
 done:
